import java.awt.Graphics;

/**
 * Cette classe dfinit les blocs qui sont utiliss par les pices du jeu
 * et la grille du jeu se sert de sa mthode static paint(Graphics g).
 * Les constantes X0 et Y0 sont l'abscisse et l'ordonne du centre du
 * repre dans lequel on va les afficher. Il s'agit d'un repre traditionnel
 * dans lequel l'axe des y croit vers le haut, et non pas vers le bas comme
 * dans la classe Graphics de l'API. Les blocs ne s'occupent pas des couleurs,
 * celles-ci sont grs par les classe Piece et Grid.
 */

class Block {
	
	// Champs de classe
	/** Taille du bloc en pixel. */
	public static final int SIZE = 18;
	/** Abscisse du centre du repre en pixel. */
	public static final int X0 = 10;
	/** Ordonne du centre du repre en pixel. */
	public static final int Y0 = X0 + Grid.ROWS * SIZE;

	// Champs de variable
	/** Abscisse du bloc en nombre de colonnes. */
	private int x;
	/** Ordonne du bloc en nombre de lignes. */
	private int y;

	// Constructeur
	public Block(int i, int j) {
		x = i;
		y = j;
	}

	// Mthodes
	/** Afficher un pseudo-bloc de coordonnes (x,y)dans le Graphics g. */
	public static void paint(Graphics g, int x, int y) {
			g.drawRect(X0+x*SIZE,Y0-(y+1)*SIZE,SIZE-2,SIZE-2);
			g.fillRect(X0+x*SIZE+2,Y0-(y+1)*SIZE+2,SIZE-2,SIZE-2);
		}

	/** Affich un bloc dans le Graphics g. */
	public void paint(Graphics g) {paint(g,x,y);}

	/** Dplacer un bloc de horizontal colonnes et de vertical lignes. */
	public void move(int horizontal, int vertical) {
		x += horizontal;
		y += vertical;
	}

	
	// Accesors
	public int getX() {return x;}
	public int getY() {return y;}

	// Mutators
	public void setX(int i) {x = i;}
	public void setY(int i) {y = i;}
	
}

